%% BMWLinearization
% A simple model, assuming level road, is:
%    dv/dt = (Fnet - cD*v^2 - Froll)/m := f(v,Fnet)

%% Model Parameters
m = 2085;       % Vehicle mass, kg
cD = 0.4;       % Drag coefficient, N*sec^2/m^2
Froll = 228;    % Rolling resistance, N

%% Equilibrium Points
% Solve the following for an equilibrium point:
%    0 = Fnet - cD*v^2 - Froll
% Note: 1m/s ~= 2.24 mph.
Fnet_eqA = 400;
v_eqA = sqrt( (Fnet_eqA-Froll)/cD );

v_eqB = 29;
Fnet_eqB = cD*v_eqB^2+Froll;

%% Simulations: Trim Conditions

% Sim at Eq Point A
Tf = 10;
v0 = v_eqA;
Fnet_data = [0 Fnet_eqA; Tf Fnet_eqA];
sim('BMWSimulation',Tf)
t_A = t;
Fnet_A = Fnet;
v_A = v;

% Sim at Eq Point B
Tf = 10;
v0 = v_eqB;
Fnet_data = [0 Fnet_eqB; Tf Fnet_eqB];
sim('BMWSimulation',Tf)
t_B = t;
Fnet_B = Fnet;
v_B = v;

figure(1);
plot(t_B,Fnet_B,'b',t_A,Fnet_A,'r--');
xlabel('Time (sec)');
ylabel('F_{net} (N)');
%ylim([390 580]);
ylim([0.95*Fnet_eqA 1.05*Fnet_eqB]);
legend('Eq. Pt. B','Eq. Pt. A','Location','East');
if exist('garyfyFigure','file') && exist('savefig','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    garyfyFigure
    %matlab2tikz('BMWTrim_Input.tikz','width','\figurewidth');
    savefig('BMWTrim_Input',1,'pdf')
end

figure(2);
plot(t_B,v_B,'b',t_A,v_A,'r--');
xlabel('Time (sec)');
ylabel('Velocity (m/s)');
%ylim([20 30]);
ylim([0.95*v_eqA 1.05*v_eqB]);
if exist('garyfyFigure','file') && exist('savefig','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    garyfyFigure
    %matlab2tikz('BMWTrim_Output.tikz','width','\figurewidth');
    savefig('BMWTrim_Output',2,'pdf')
end

%% Taylor Series for Drag Force
% Linear approximation for drag force is:
%   cD*v^2 ~= cD*veq^2 + (2*cD*veq)*(v-veq);
NTS = 100;
vTS = linspace(10,40,NTS);

figure(3);
plot(vTS,cD*vTS.^2,'b',vTS,cD*v_eqB^2 + (2*cD*v_eqB)*(vTS-v_eqB),'r--')
xlabel('Velocity (m/s)')
ylabel('Drag force, c_D v^2 (N)');
legend('Nonlinear','Linear Taylor Series','Location','North');
grid
if exist('garyfyFigure','file') && exist('savefig','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    garyfyFigure
    savefig('DragTaylorSeries',3,'pdf')
end

%% Linearization
% Taylor series expansion of f(v,Fnet) around the eq point gives:
%   d(delta_v)/dt ~=  [(-2*cD*veq)*delta_v + delta_Fnet]/m
% where the deviations are defined as:
%   delta_v = v-veq
%   delta_Fnet = Fnet-Fneteq
% This yields the linear ODE:
%   d(delta_v)/dt + a0*delta_v = b0*delta_Fnet
a0_A = 2*cD*v_eqA/m;
b0_A = 1/m;
G_A = ss(-a0_A,b0_A,1,0);

a0_B = 2*cD*v_eqB/m;
b0_B = 1/m;
G_B = ss(-a0_B,b0_B,1,0);

%% Simulations (not at trim)

% Linear/Nonlinear Sim near Eq Point A
Tf = 10;
dv0 = -1;
v0 = v_eqA+dv0;

tF = linspace(0,Tf,200);
dFnet = -140*sin(2*tF(:));
Fnet_data = [tF(:)   Fnet_eqA+dFnet];
sim('BMWSimulation',Tf)
t_NLNear = t;
Fnet_NLNear = Fnet;
v_NLNear = v;

% Linear Sim near Eq Pt A
[dv_Near,t_LNear] = lsim(G_A,dFnet,tF,dv0);
v_LNear = dv_Near+v_eqA;

figure(4);
plot(t_NLNear,v_NLNear,'b',t_LNear,v_LNear,'r-.')
%    [0 Tf],v_eqA*[1 1],'k:');
xlabel('Time (sec)');
ylabel('Velocity (m/s)');
legend('Nonlinear','Linear','Location','North');
if exist('garyfyFigure','file') && exist('savefig','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    garyfyFigure
    savefig('BMWLin_Near',4,'pdf')
end

% Linear/Nonlinear Sim far from Eq Point A
Tf = 10;
dv0 = -10;
v0 = v_eqA+dv0;

tF = linspace(0,Tf,200);
dFnet = -140*sin(2*tF(:));
Fnet_data = [tF(:)   Fnet_eqA+dFnet];
sim('BMWSimulation',Tf)
t_NLFar = t;
Fnet_NLFar = Fnet;
v_NLFar = v;

% Linear Sim near Eq Pt A
[dv_Far,t_LFar] = lsim(G_A,dFnet,tF,dv0);
v_LFar = dv_Far+v_eqA;

figure(5);
plot(t_NLFar,v_NLFar,'b',t_LFar,v_LFar,'r-.');
xlabel('Time (sec)');
ylabel('Velocity (m/s)');
if exist('garyfyFigure','file') && exist('savefig','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    garyfyFigure
    savefig('BMWLin_Far',5,'pdf')
end
